CON

  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000

  'Parallax Serial Terminal Control Character Constants
  HM =  1 'HM: Home Cursor       
  PC =  2 'PC: Position Cursor In X, Y
  ML =  3 'ML: Move Cursor Left          
  MR =  4 'MR: Move Cursor Right         
  MU =  5 'MU: Move Cursor Up          
  MD =  6 'MD: Move Cursor Down
  BP =  7 'BP: Beep Speaker
  BS =  8 'BS: Backspace
  TB =  9 'TB: Tab
  LF = 10 'LF: Line Feed
  CE = 11 'CE: Clear To End Of line     
  CB = 12 'CB: Clear Lines Below
  CR = 13 'CR: Carriage Return          
  PX = 14 'PX: Position Cursor In X         
  PY = 15 'PY: Position Cursor In Y
  CS = 16 'CS: Clear Screen
  SP = 32 'Space

  'Master And Slave Communication Constants
  MASTER_RX = 0 'Master RX Pin
  MASTER_TX = 1 'Master TX Pin
  SLAVE_RX = MASTER_TX 'Slave RX Pin
  SLAVE_TX = MASTER_RX 'Slave TX Pin
  MASTER_COMM_MODE = 0
  SLAVE_COMM_MODE = 0
  COMM_BAUD_RATE = 115_200

  MAX_LEN_NUM_STR = 11 'For NumericString 10 Characters + Zero Terminator
  MAX_LEN_TX_STR = 22 'For NewTxString 21 Characters + Zero Terminator

  COUNTER_MAX = 2_147_483_648
  
VAR

  LONG DataField1
  LONG DataField2
'  LONG LongValue1  
'  LONG LongValue2

  BYTE NumericString[MAX_LEN_NUM_STR]
  BYTE NewTxString[MAX_LEN_TX_STR]
  LONG Counter  

OBJ

  Serial : "FullDuplexSerial"
  Master : "InterComm"
  Slave : "InterComm"
  
PUB Main

  Serial.Start(31, 30, 0, 115_200) 

  WAITCNT(CLKFREQ * 2 + CNT)  

  DisplayPreamble

  InitializeMasterAndSlave

  REPEAT WHILE Counter <> COUNTER_MAX

    IF Counter // 4 == 0

      MasterTransmitSlaveReceive(Counter++, Counter++)

    ELSE

      SlaveTransmitMasterReceive(Counter++, Counter++)

  DisplayConclusion

PUB MasterTransmitSlaveReceive(LongValue1, LongValue2)

  Serial.Str(STRING("The Master Is Now Sending Two Long Values To The Slave!"))
  Serial.Tx(CR)

  REPEAT WHILE Master.IsHandshakeInitiated == TRUE

  MasterTransmitTwoLongsToSlave(LongValue1, LongValue2)

  REPEAT WHILE Slave.IsNewDataAvailable == FALSE

  Slave.GetDataFields(@DataField1, @DataField2)

  IF DataField1 <> LongValue1 OR DataField2 <> LongValue2

    DisplayErrorAndAbort

  ELSE

    DisplayResults  

PUB SlaveTransmitMasterReceive(LongValue1, LongValue2)

  Serial.Str(STRING("The Slave Is Now Sending Two Long Values To The Master!"))
  Serial.Tx(CR)

  REPEAT WHILE Slave.IsHandshakeInitiated == TRUE

  SlaveTransmitTwoLongsToMaster(LongValue1, LongValue2)

  REPEAT WHILE Master.IsNewDataAvailable == FALSE
  
  Master.GetDataFields(@DataField1, @DataField2)

  IF DataField1 <> LongValue1 OR DataField2 <> LongValue2

    DisplayErrorAndAbort

  ELSE

    DisplayResults

PUB DisplayPreamble

  Serial.Tx(CS)
  Serial.Rxflush

  Serial.Str(STRING("######  InterComm Object Demonstration  ######"))
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Str(STRING("The InterComm Object Now Includes:"))
  Serial.Tx(CR)
  Serial.Str(STRING("CRC Error Checking, Handshaking, And Minimal FIFO!"))
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Str(STRING("It Is Better, Faster, Stronger, And More Dependable Then All Previous Versions!"))
  Serial.Tx(CR)
  Serial.Str(STRING("Plus It Is Much Easier To Interface Then All Previous Versions!"))
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Str(STRING("This Is A Propeller To Propeller Communication Simulation"))
  Serial.Tx(CR)
  Serial.Str(STRING("Which Demonstrates How To Interface The InterComm Object!"))
  Serial.Tx(CR)
  Serial.Str(STRING("The Demonstration Also Includes Transmission/Reception Comparisons"))
  Serial.Tx(CR)
  Serial.Str(STRING("To Help Verify That Your Transmissions And Receptions Are 100% Accurate!"))
  Serial.Tx(CR)  
  Serial.Tx(CR)
  Serial.Str(STRING("This Software Can Be Modified To Meet Most The Needs And Requirements Of"))
  Serial.Tx(CR)
  Serial.Str(STRING("CNC Communication Operations Such As G-Code Transmittal And Reception!"))
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Str(STRING("The InterComm Object Demonstration Will Commence In Approximately 50 Seconds!"))

  WAITCNT(CLKFREQ * 50 + CNT)

  Serial.Tx(CS)

PUB DisplayResults

  Serial.Tx(CR)
  Serial.Str(STRING("Results Of The Transmission: "))
  Serial.Tx(CR)
  Serial.Str(STRING("Value For DataField1: "))
  Serial.Dec(DataField1)
  Serial.Tx(CR)
  Serial.Str(STRING("Value For DataField2: "))
  Serial.Dec(DataField2)

  Serial.Tx(CR)
  Serial.Tx(CR)

PUB DisplayErrorAndAbort

  Serial.Tx(CR)
  Serial.Str(STRING("ERROR: Write And Read Operations Are Colliding!"))
  Serial.Tx(CR)
  Serial.Str(STRING("Now Aborting"))

  ABORT

PUB DisplayConclusion

  Serial.Tx(CS)
  Serial.Rxflush

  Serial.Str(STRING("If You Have Reached This Point Without Shutting Down"))
  Serial.Tx(CR)
  Serial.Str(STRING("Then You Must Have Been Determined To See It Fail!"))
  Serial.Tx(CR)
  Serial.Tx(CR)
  Serial.Str(STRING("The InterComm Object Demonstration Is Now Complete!"))
  
PUB InitializeMasterAndSlave

  IFNOT Master.Start(MASTER_RX, MASTER_TX, MASTER_COMM_MODE, COMM_BAUD_RATE)

    Serial.Str(STRING("Master Initialization Failed"))
    Serial.Tx(CR)

    ABORT

  IFNOT Slave.Start(SLAVE_RX, SLAVE_TX, SLAVE_COMM_MODE, COMM_BAUD_RATE)

    Serial.Str(STRING("Slave Initialization Failed"))
    Serial.Tx(CR)

    ABORT

  WAITCNT(CLKFREQ * 2 + CNT)
  
PUB MasterTransmitTwoLongsToSlave(Value1, Value2)

  ConvertLongsIntoTxString(Value1, Value2)
  Master.UpdateTxString(@NewTxString)

PUB SlaveTransmitTwoLongsToMaster(Value1, Value2)

  ConvertLongsIntoTxString(Value1, Value2)
  Slave.UpdateTxString(@NewTxString)

PRI ConvertLongsIntoTxString(Value1, Value2)

  RESULT := @NewTxString
  NewTxString~

  BYTEMOVE(RESULT + STRSIZE(RESULT), ConvertDecimalToString(Value1), MAX_LEN_NUM_STR + 1)
  BYTEMOVE(RESULT + STRSIZE(RESULT), STRING(":"), STRSIZE(STRING(":")) + 1)
  BYTEMOVE(RESULT + STRSIZE(RESULT), ConvertDecimalToString(Value2), MAX_LEN_NUM_STR + 1)

PRI ConvertDecimalToString(DecimalValue) | Index, Divisor, ZeroPad   

  BYTEFILL(@NumericString, 0, MAX_LEN_NUM_STR)

  Index := 0  

  Divisor := 1_000_000_000
  ZeroPad~

  REPEAT 10
  
    IF (DecimalValue => Divisor)
    
      NumericString[Index++] := (DecimalValue / Divisor + "0")
      DecimalValue //= Divisor
      ZeroPad~~
      
    ELSEIF ZeroPad or (Divisor == 1)
    
      NumericString[Index++] := "0"
    Divisor /= 10 

  return @NumericString
  